# main.py
import machine
import neopixel
import time

# Pins
PIN_NP = 5
PIN_Y= 0
PIN_G= 4
PIN_B= 2
# NeoPixel
NUM_LED = 16
# y - Mode
MODE_MAX = 5
MODES = [ "off", "on", "color_chase", "rainbow", "strobe" ]
MODE_STEP = 1
# g - Color
COLOR_MAX = 255
COLOR_STEP = 5
# b - Brightness
BRIGHTNESS_MAX = 250
BRIGHTNESS_STEP = 50
# Delay
DELAY = 0.1
DELAY_IRQ = 0.5
DELAY_CHASE = 0.05
DELAY_STROBE = 0.2

light = None
state = None

def handle_y(pin):
    global light, state
    if state is None:
        state = machine.disable_irq()
        light.set_fn("mode")

def handle_g(pin):
    global light, state
    light.set_fn("color")

def handle_b(pin):
    global light, state
    light.set_fn("brightness")

class Light:
    def __init__(self):
        self.mode = 0
        self.color = 0
        self.brightness = 0
        self.np = neopixel.NeoPixel(machine.Pin(PIN_NP), NUM_LED)
        self.sw_y = machine.Pin(PIN_Y, machine.Pin.IN, machine.Pin.PULL_UP)
        self.sw_g = machine.Pin(PIN_G, machine.Pin.IN, machine.Pin.PULL_UP)
        self.sw_b = machine.Pin(PIN_B, machine.Pin.IN, machine.Pin.PULL_UP)
        self.sw_y.irq(trigger=machine.Pin.IRQ_FALLING, handler=handle_y)
        self.sw_g.irq(trigger=machine.Pin.IRQ_FALLING, handler=handle_g)
        self.sw_b.irq(trigger=machine.Pin.IRQ_FALLING, handler=handle_b)

    def set_fn(self, fn):
        if fn == "mode":
            self.mode = (self.mode + MODE_STEP) % MODE_MAX
            print("set fn {} to {}".format(fn, MODES[self.mode]))
        elif fn == "color":
            self.color = (self.color + COLOR_STEP) % COLOR_MAX
            print("set fn {} to {}".format(fn, self.color))
        elif fn == "brightness":
            self.brightness = (self.brightness + BRIGHTNESS_STEP) % BRIGHTNESS_MAX
            print("set fn {} to {}".format(fn, self.brightness))
        else:
            print("invalid fn {}".format(fn))

    def set_color(self, color):
        for i in range(NUM_LED):
            self.np[i] = self.wheel(color)
        self.np.write()

    def wheel(self, pos):
        if pos < 0 or pos > 255:
            return (0, 0, 0)
        if pos < 85:
            return (255 - pos * 3, pos * 3, 0)
        if pos < 170:
            pos -= 85
            return (0, 255 - pos * 3, pos * 3)
        pos -= 170
        return (pos * 3, 0, 255 - pos * 3)

    def start(self):
        global state
        old_mode = self.mode
        old_color = self.color
        old_brightness = self.brightness
        i = 0
        j = 0
        while True:
            if state:
                time.sleep(DELAY_IRQ)
                machine.enable_irq(state)
                state = None
            if not self.mode == old_mode:
                print("mode: {}, old_mode: {}".format(MODES[self.mode], MODES[old_mode]))
            if MODES[self.mode] == "off":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    self.set_color(-1)
                time.sleep(DELAY)
            # Off
            elif MODES[self.mode] == "on":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    self.set_color(self.color)
                if not self.color == old_color:
                    old_color = self.color
                    self.set_color(self.color)
                time.sleep(DELAY)
            # On
            elif MODES[self.mode] == "color_chase":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    self.set_color(-1)
                    i = 0
                    j = 0
                    k = 0
                if not self.color == old_color:
                    old_color = self.color
                self.np[i] = self.wheel(self.color)
                self.np[(i+1) % NUM_LED] = self.wheel(self.color)
                self.np[(i+2) % NUM_LED] = self.wheel(self.color)
                self.np.write()
                self.np[i] = self.wheel(-1)
                i = (i + 1) % NUM_LED
                time.sleep(DELAY_CHASE)
            # Rainbow
            elif MODES[self.mode] == "rainbow":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    i = 0
                    j = 0
                rc_index = (i * 256 // NUM_LED) + j
                self.np[i] = self.wheel(rc_index & 255)
                i = (i + 1)
                if i == NUM_LED:
                    i = 0
                    self.np.write()
                    j = (j + 1) % 255
            # Strobe
            elif MODES[self.mode] == "strobe":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    j = 0
                if j == 1:
                    self.set_color(self.color)
                else:
                    self.set_color(-1)
                j = (j + 1) % 2
                time.sleep(DELAY)
            else:
                print("else")


def main():
    print("")
    global light
    light = Light()
    light.start()


if __name__ == "__main__":
    main()